/* PulseSensor_IBI Distribution
Based on PulseSensor Amped HRV Poincare Plot by Joel Murphy, 2013
Modifies by Paul Christensen, 2014
Use this with "PulseSensor_FastReplay" Arduino code and the EasyPulse Sensor hardware.

This code plots interbeat interval for serial data transmitted from Arduino + EasyPulse Sensor hardware.  Interbeat intervals from the immediately previous
100 beats are grouped in 64 ms time segments, and the number in each segment is plotted as a percent of the 100 beats. After 100 beats have been recorded, the average pulse
rate is displayed, the most recent beat is added to the 100-beat data array and the oldest data point in the array is discarded
 
key press commands include:
  press 'S' to save a picture of the Processing window (JPG image saved in Sketch folder).  File name format is "Month and Day_hour-minute"
  press 'C' to clear the Poincare Plot
  press '1' to display y axis at 100% full scale  (magnify y amplitude by 1)
  press '2' to desplay yaxis at 50% full scale   (magnify y amplitude by 2)
  press 'F' to initiate audio playback through an earbud of heartbeat data at 8X speed
  
This code was created by Paul Christensen, Feb 2014, and released into the public domain.  Suitability for particular applictions is not guaranteed.
*/

import processing.serial.*;  // Serial library makes it possible to talk to Arduino
PFont font;                  // we will use text in this sketch
Serial port;                 // instantiate the Serial port

int IBI;                  // length of time between heartbeats in milliseconds (updated in serialEvent)
int[] PPG;                // array of live PPG datapoints
int[] beatTimeX;          // array of X coordinates of Poincare Plot
int[] beatTimeY;          // array of Y coordinates of Poincare Plot
int beatTimeTotal;        // Sum of beat times for array of beatTimeX
int numPoints = 100;      // size of coordinate arrays. ths sets number of displayed datapoints
int[] range;              //each 64 ms time segments that extend from the minimum to the maximum interbeat interval (IBI)
int graphMag;             // scale factor for the vertical plot axis
// initializing flags here
boolean pulse = false;    // pulse is made true in serialEvent when arduino sends new IBI value
boolean makeLine = false; // press 'L' to toggle a trace the last 20 dots on the Poincare Plot

void setup() {                     // do all the sett'n up in the setup
size(800,650);                     // Stage size
frameRate(60);
beatTimeX = new int[numPoints];    // these two arrays hold the IBI Plot data
beatTimeY = new int[numPoints];    // size of numPoints determines number of displayed points
PPG = new int[150];                // PPG array that that prints heartbeat waveform

range = new int[50];              //IBI data will be divided into no more than 50 ranges
graphMag = 1;                      // Graph will initially plot 0 - 100%

for (int i=0; i<150; i++){
 PPG[i] = height/2+15;             // initialize PPG widow with dataline at midpoint
}

font = loadFont("Arial-BoldMT-36.vlw");
textFont(font);                    // general house-keeping in Processing
textAlign(CENTER);                 // text and shapes will be referenced to their center point
rectMode(CENTER);

// FIND AND ESTABLISH CONTACT WITH THE SERIAL PORT
println(Serial.list());          // print a list of available serial ports
port = new Serial(this, Serial.list()[1], 115200); // Set the right COM port and baud rate here
port.bufferUntil('\n');          // arduino will end each data packet with a carriage return 
port.clear();                    // flush the Serial buffer
}  // END OF SETUP


void draw(){
   background(0);
//  DRAW THE BACKGROUND ELEMENTS AND TEXT
   noStroke();
   textFont(font); 
   fill(255,253,248);                        // eggshell white  
   text("Interval between heartbeats",width/2-50,40);  // title 
   rect(width/2-50,height/2+15,550,550);     // draw Poincare Plot window
   rect(width-85,(height/2)+15,150,550);     // draw the Pulse Sensor data window   
   fill(200);                                // draw the Plot coordinate values in grey
   text("0",40,height-25);                 // origin, scaled in mS
   if(graphMag == 1){

      for (int i=20; i<=100; i+=20){         // print y axis values
     text(i, 40,map(i,0,100,615,75));
   }
   }
   else if (graphMag==2){
   for (int i=10; i<=50; i+=10){         // print y axis values
     text(i, 40,map(i,0,50,615,75));
   }
   } 
   
   for (int i=500; i<=1500; i+=500){         // print  x axis values
     text(i, 75+map(i,0,1500,0,550), height-10);
   }
//   stroke(250,30,250);                       // draw gridlines in purple
//   for (int i=0; i<1500; i+=100){            // draw grid lines on axes
//     line(75,map(i,0,100,614,26),85,map(i,0,1500,614,26)); //y axis
//     line(75+map(i,0,1500,0,549),height-35,75+map(i,0,1500,0,549),height-45); // x axis
//   }
//   noStroke();
   fill(255,253,10);                                    // print axes legend in yellow, for fun
   text("ms", 75+map(750,0,1500, 0, 550), height-10);    // n is the most recent IBI value
   text("%",40,map(750,0,1500,615,75));               // n-1 is the one we got before n

//    DRAW THE INTERVAL PLOT   
  if (pulse == true){                    // check for new data from arduino
    pulse = false;                       // drop the pulse flag. it gets set in serialEvent
      IBI = constrain(IBI,0,1500);
      range[IBI>>6] ++;                  //
      range[beatTimeX[99]>>6]--;
   
    for (int i=numPoints-1; i>0; i--){   // shift the data in n and n-1 arrays 
//      beatTimeY[i] = beatTimeY[i-1];
      beatTimeX[i] = beatTimeX[i-1];     // shift the data point through the array
    }
 //     beatTimeY[0] = beatTimeX[1];       // toss the old n into the n-1 spot
      beatTimeX[0] = IBI;                // update n with the current IBI value
    }

  fill(0,0,255);                         //  draw a history of the data points as blue dots
  for (int i=1; i<25; i++){
    float  x = map(i<<6,0,1500,75,600);  // scale the data to fit the screen
    float  y = map(range[i],0,100/graphMag,615,25);  // invert Y so it looks normal
    ellipse(x,y,20,20);  
}

 // Display average heart rate
beatTimeTotal = 0;
 for (int i=0; i<numPoints-1; i++){
   beatTimeTotal = beatTimeTotal + beatTimeX[i];
 }

 if (beatTimeX[99] > 10){        //Check that the 100 data point has arrived
   float flBPM = 6000000/beatTimeTotal;
   int BPM = int(flBPM);     //convert to integer format for display
    noStroke();
   textFont(font); 
   fill(255,253,248);                        // eggshell white  
   text("Avg BPM:",width-180,40);  // label 
   text(BPM,width-75,40);     //value
}

//   GRAPH THE PULSE SENSOR DATA 
 stroke(250,0,0); 
 noFill();
 // use red for the pulse wave
  beginShape();                                         // beginShape is a fast way to draw lines!
  for (int i=1; i<PPG.length-1; i++){                   // scroll through the PPG array
    int x = width-160+i;
    //x = 640-i;
    int y = PPG[i];                                         
    vertex(x,y);                                        // set the vertex coordinates
  }
  endShape();                                           // connect the vertices;
  noStroke();
 }  //END OF DRAW
 





